
% This function is an approximate implemenation of the ELP histogram as
% proposed in:
% Hamid Tizhoosh, Morteza Babaie, "Representing Medical Images with Encoded Local Projections"
% IEEE Transactions on Biomedical Engineering, 2018

% Hamid R.Tizhoosh, Morteza Babie
% Januray 2018
% 1imia Lab, University of Waterloo
% http://1imia.ueaterloo.ca
% This code can only be used for research purposes



function [Out] = getELPN(IMG,w,step,homoThr,DETACHflag)
tic
% Inputs: IMG (normlaized and, if necessary, undersampled image)
%         w: windows size (10 means 10x10);
%         step: stripe size
%           homoThr: threshold for homogeneity
% Outputs: H (ELP histogram)
% convert the image to gray levels if necessary
if size(IMG,3)>3
    Warning = 'image dimensions are invalid, extra dimensions will be discarded'
    IMG=IMG(:,:,1:3);
end
if size(IMG,3)>1
    IMG=rgb2gray(IMG);
end

% Extract the gradient direction for the entire image
% (as approximation for calculation of all local projections to detect the anchor projection)
[~,Gdir] = imgradient(IMG);

% Get the image dimensions
[rows, cols] = size(IMG);
allBinaryCodes=(zeros([rows-w 4*(cols-w)]))/0;

% initialize the number of projection words(=number of local windows)
numProjectionWords = 0;

% initialize counters for the projection fingerprint
mm = 1;
nn = 1;
offset=0;
cnt=0;
% scan the entire image to construct the ELP histogram
for i=1+offset:step:rows-w-offset
    for j=1+offset:step:cols-w-offset
        % get a subimage
        subIMG = IMG(i:i+w,j:j+w);
        
        % Is the current window heterogenous enough?
        degHomogen = calcHomogeneity(subIMG);        
        
        % process the window if heterogenous enough
        if degHomogen<homoThr
            % increment the number of local windows
            numProjectionWords = numProjectionWords + 1;
            
            % detect the anchor projection
            cnt=cnt+1;
            anchorProjection(cnt) = median(median(Gdir(i:i+w,j:j+w)));
            % attach three other projections to the anchor projection
            whichIsMax = round(anchorProjection(cnt) -[0 45 90 125]);
            
            %selected based on anchor
            whichIsMax(whichIsMax<=0) = 180 + whichIsMax(whichIsMax<=0);%correct the...
            
            % negative directions by adding 180
            Degs = whichIsMax;
            
            % now calculte the Radon projections
            R = radon(subIMG,Degs);
            
            % eliminate the zero-paddings in the sinogram
            Cut = ceil((size(R,1)-w)/2);
            R = R(Cut:end-Cut,:);
            
            % approximate the derivatives of all projections
            DR = (R(2:end,:) - R(1:end-1,:));
            
            % binarize the derivates via MinMax approach
            DRbin = DR(2:end,:) - DR(1:end-1,:) > 0;
            
            % convert binary vectors to integers
            if mm==355
                aa=1;
            end
            allBinaryCodes(mm,nn:(cols-w):end) = (bi2de(DRbin'))';
            
            % Count the frequncies
          
            nn = nn + 1;
        end
    end
    nn = 1;
    mm = mm + 1;
end


%calculate the histogram
[Rows,Cols] =size(double(allBinaryCodes));
Cols=Cols/4;
nn=1;% Number of vertical sections to create subimags
mm=1;% Number of Horizental sections
Hist_MAX=255;%2^floor(log2(double(max(max(ELP_Train{1})))))+1;% Maximum of beans value
Hist_size=256;% The size of the outpot histogram
Hist_step=floor(linspace(1,Hist_size,Hist_MAX+1));
fn=floor(Rows/nn);%Each sub image Height
fm=floor(Cols/mm);%Each sub image Weith
if DETACHflag==0
    Out=(zeros(1,Hist_size*nn*mm));
    %predefine outpot variable
    NormOut=(zeros(1,Hist_size*nn*mm));
    %predefine Normal outpot variable
else
    Out=(zeros(1,4*Hist_size*nn*mm));
    %predefine outpot variable
    NormOut=(zeros(1,4*Hist_size*nn*mm));
    %predefine Normal outpot variable
end
SubImgHist=(zeros(4,nn*mm*Hist_size,1));
%Pre-define variable to Save each subImg Histogram
    
    TmpELP=double(allBinaryCodes)+1;
    % A temperory matrix to 1eep row ELP information 
    %add one to increase all beans
    %this step helps to reserve zeros for Hemogenity detection
    %T=T(:,1:4*Cols) ;
    for Dir_ind=1:4% 4 is the number of direction
        Each_Dir_TmpELP=TmpELP(:,(Dir_ind-1)*Cols+1:Dir_ind*Cols);
        %Save each direction seperately to Count them
        for ww=1:nn 
            for xx=1:mm
                HIndex=((ww-1)*mm+xx);
                SubImgDirT=Each_Dir_TmpELP((ww-1)*fn+1:ww*fn,(xx-1)*fm+1:xx*fm);
                %Save each subimage per specific direction
                Vect_SubImg=SubImgDirT(:);
                %Vectorise the matrix to count them
                SubImgHist(Dir_ind,(HIndex-1)*Hist_size+1:HIndex*Hist_size)=histc(Vect_SubImg,Hist_step);   
                %Create Histogram per each subimage and each direction 
            end
        end
    end
    %Out(1,1)=Out(1,1)-Z;
    if DETACHflag==0
    Out(1,:)=sum(SubImgHist(:,:,1));
    else
    Out(1,:)=reshape(SubImgHist(:,:,1),1,[]);
    end
    NormOut(1,:)=bsxfun(@rdivide, Out(1,:), sqrt(sum(Out(1,:).^2)) + eps('single'));
    %Out(1,:)=Out(1,:)/norm(Out(1,:));
    %Out(1,:)=(Out(1,:)-mean(Out(1,:)))/std(Out(1,:));
toc-tic
    function [H] = calcHomogeneity(I)
        %CALCHOMOGENEITY Summary of this function goes here
        %   Detailed explanation goes here
        
        m = median(I(:));
        dev = (I-m).^2;
        H = 1-2^(-8)*sqrt(sum(sum(dev)));
        
    end
end

